/* -*- mode: c++; c-basic-offset: 3; -*- */
#include "bashpat_wrap.hh"
#include "bash_pattern.h"

struct bash_pattern_t {
   enum {
      BASH_PATTERN_STRING_LITERAL,
      BASH_PATTERN_ANY_STRING,
      BASH_PATTERN_ANY_CHAR,
      BASH_PATTERN_ALTERNATIVES
   } kind;
   bash_pattern *next;
};

struct bash_pattern_string_literal {
   bash_pattern_t node;
   size_t len;
   char *text;
};

using namespace std;

bashpat_wrap::bashpat_wrap(const string& pat) 
   : bp_struct(NULL)
{
   if (!pat.empty()) compile(pat);
}

bashpat_wrap::~bashpat_wrap(void) {
   if (bp_struct) bash_pattern_free(bp_struct);
}

void bashpat_wrap::compile(const string& pat) {
   bp_struct = bash_pattern_compile(pat.c_str());
}

bool 
bashpat_wrap::match(const std::string& str) const {
   return bash_pattern_matches(bp_struct, str.c_str());
}

std::string
bashpat_wrap::literal(void) const {
   if (!bp_struct || 
       bp_struct->kind != bash_pattern_t::BASH_PATTERN_STRING_LITERAL) return "";
   const bash_pattern_string_literal* string_literal = 
      reinterpret_cast<const bash_pattern_string_literal*>(bp_struct);
   if (!string_literal->text) return "";
   return string(string_literal->text, string_literal->len);
}
